import logging
import os
import RainbowMonitoringSDK.exporters as exporters
import RainbowMonitoringSDK.probes as probes
import time

from RainbowMonitoringSDK.utils.basics import time_to_seconds


class Controller(object):
    """
    The controller is responsible to orchestrate the execution of:
    - Sensing Units (Metric Collectors)
    - Dissemination Units
    """
    sensing_units: dict = {}
    dissemination_units: dict = {}
    configs: dict = {}

    def __init__(self, configs: dict = {}):
        self.configs = configs
        node_id = os.getenv('NODE_ID', self.configs.get('node_id', None))
        if not node_id: raise Exception("The node id is not defined")
        os.environ['NODE_ID'] = node_id

    def instantiate_sensing_units(self):
        """
        This function instantiates the sensing units and initialize their parameters
        :return:
        """
        res = self.configs['sensing-units'] if 'sensing-units' in self.configs else {}
        for i in res:
            metric_collector_class = getattr(probes, i, None)
            if metric_collector_class:
                print("Sensing Unit %s is instantiating" % i)
                current_conf = self.configs['sensing-units'][i]
                current_conf['periodicity'] = time_to_seconds(current_conf['periodicity'])
                self.sensing_units[i] = metric_collector_class(**current_conf)

    def instantiate_dissemination_units(self):
        """
        It instantiates the dissemination channels and injects their configuration parameters
        :return:
        """
        res = self.configs['dissemination-units'] if 'dissemination-units' in self.configs else {}
        for i in res:
            extractor_class = getattr(exporters, i, None)
            if extractor_class:
                print("Dissemination Unit %s is instantiating" % i)
                self.dissemination_units[i] = extractor_class(**self.configs['dissemination-units'][i])

    def start_sensing_units(self):
        """
        Starts, if it is necessary, the threads of the sensing units
        :return:
        """
        for i in self.sensing_units:
            print("Sensing Unit %s is starting" % i)
            self.sensing_units[i].activate()

    def start_dissemination_units(self):
        """
        Starts, if it is necessary, the dissemination channels connections e.g. starting Restful server, connection with kafka, etc
        :return:
        """
        for i in self.dissemination_units:
            print("Dissemination Unit %s is starting" % i)
            self.dissemination_units[i].activate()

    def start_control(self):  # TODO update it to encaptulate Queue
        """
        The main control loop of the monitoring system.
        In the loop, system captures one-by-one all metrics from metric collectors, combines them and
        disseminates them to all channels.
        :return:
        """
        running = True
        while running:
            try:
                res = {}
                for i in self.sensing_units:
                    sensing_unit = self.sensing_units[i]
                    metrics = sensing_unit.get_metrics()
                    res.update({i: metrics})
                    # sensing_unit.update()  # TODO Add adaptiveness functionality to update
                for i in self.dissemination_units:
                    self.dissemination_units[i].update(res)
                periodicity = self.configs['sensing-units']['general-periodicity']
                time.sleep(time_to_seconds(periodicity))
            except Exception as ex:
                logging.error("Error at start_control",
                              exc_info=True)
                print(ex)
        exit(0)



import unittest

from RainbowMonitoringSDK.probes.probelib.AppMetricsProbe import AppMetricsProbe
from unittest.mock import patch


class TestAppMetricsProbe(unittest.TestCase):

    def setUp(self):
        self.configs = {"sources":["/test"]}
        self.app_probe = AppMetricsProbe(5, **self.configs)

    def test_init(self):
        self.assertEqual(self.app_probe.get_name(),"AppLevelMetrics")
        self.assertEqual(self.app_probe.get_periodicity(), 5)
        self.assertEqual(self.app_probe.config, self.configs)

    @patch.object(AppMetricsProbe, 'retrieve_metrics')
    def test_collect(self,retrieve_metrics):
        vals = [
            {"name": "name-1", "units": "unit-1", "desc": "desc-1", "val": 1},
            {"name": "name-2", "units": "unit-2", "desc": "desc-2", "val": 2}
            ]
        retrieve_metrics.return_value = vals
        prob = AppMetricsProbe(5, **self.configs)
        prob.collect()
        self.assertEqual(["name-1", "name-2"], list(prob.metrics.keys()))

    @patch.object(AppMetricsProbe, 'retrieve_metrics')
    def test_get_metrics(self, retrieve_metrics):
        vals = [{"name": "name-1", "units": "unit-1", "desc": "desc-1", "val": 1},
            {"name": "name-2", "units": "unit-2", "desc": "desc-2", "val": 2}]
        retrieve_metrics.return_value = vals
        prob = AppMetricsProbe(5, **self.configs)
        prob.collect()
        self.assertEqual(["name-1", "name-2"], list(prob.get_metrics().keys()))



from RainbowMonitoringSDK.probes.probelib.NetDataProbe import NetDataProbe as DefaultMonitoring
from RainbowMonitoringSDK.probes.probelib.AppMetricsProbe import AppMetricsProbe as UserDefinedMetrics
from RainbowMonitoringSDK.probes.probelib.WebAppMetricsProbe import WebAppMetricsProbe as WebUserDefinedMetrics
from RainbowMonitoringSDK.probes.utils import RainbowUtils, NetDataController
from RainbowMonitoringSDK.probes.Probe import Probe
from RainbowMonitoringSDK.probes.Metric import Metric, DiffMetric, CounterMetric, SimpleMetric, TimerMetric



import unittest
import uuid
from datetime import datetime
from queue import Queue
from unittest.mock import patch

from RainbowMonitoringSDK.probes.Metric import Metric
from RainbowMonitoringSDK.probes.Probe import Probe, ProbeStatus


class TestMetric(unittest.TestCase):

    @patch.multiple(Probe, __abstractmethods__=set())
    def setUp(self):
        self.metric = Metric("metric-1", "units", "desc")  # dummy metric for test


    def test_name(self):
        self.assertEqual(self.metric.get_name(),"metric-1")
        self.metric.set_name("metric-2")
        self.assertEqual(self.metric.get_name(), "metric-2")

    def test_units(self):
        self.assertEqual(self.metric.get_units(), "units")
        self.metric.set_units("units-2")
        self.assertEqual(self.metric.get_units(), "units-2")

    def test_desc(self):
        self.assertEqual(self.metric.get_desc(), "desc")
        self.metric.set_desc("desc-2")
        self.assertEqual(self.metric.get_desc(), "desc-2")

    def test_higherIsBetter(self):
        self.assertEqual(self.metric.get_higherisbetter(), True)
        self.metric.set_higherisbetter(False)
        self.assertEqual(self.metric.get_higherisbetter(), False)
        self.assertEqual(Metric("metric-1", "units", "desc", higherIsBetter=False).get_higherisbetter(), False)


    def test_minVal(self):
        self.assertEqual(self.metric.get_minval(), None)
        self.metric.set_minval(0.0)
        self.assertEqual(self.metric.get_minval(), 0.0)
        self.assertEqual(Metric("metric-1", "units", "desc", minVal=0.0).get_minval(), 0.0)

    def test_maxVal(self):
        self.assertEqual(self.metric.get_maxval(), None)
        self.metric.set_maxval(0.0)
        self.assertEqual(self.metric.get_maxval(), 0.0)
        self.assertEqual(Metric("metric-1", "units", "desc", maxVal=0.0).get_maxval(), 0.0)

    def test_group(self):
        self.assertEqual(self.metric.get_group(), None)
        self.metric.set_group("test-group")
        self.assertEqual(self.metric.get_group(), "test-group")

    def test_val(self):
        self.assertEqual(self.metric.get_val(), None)
        self.metric.set_val(0.0)
        self.assertEqual(self.metric.get_val(), 0.0)
        self.metric.set_val(1.0)
        self.assertEqual(self.metric.get_val(), 1.0)

    def test_timestamp(self):
        self.assertEqual(self.metric.get_timestamp(), None)
        first_timestamp = datetime.now().timestamp()
        self.metric.set_timestamp(first_timestamp)
        self.assertEqual(self.metric.get_timestamp(), first_timestamp)
        second_timestamp = datetime.now().timestamp()
        self.metric.set_timestamp(second_timestamp)
        self.assertEqual(self.metric.get_timestamp(), second_timestamp)





from threading import Thread, Event
from RainbowMonitoringSDK.utils.basics import ThreadSafeDict


class Exporter(Thread):

    def __init__(self, name):
        super(Exporter, self).__init__(name=name)
        self.name = name
        # exporter's status initialized as inactive
        self.exporter_status = ExporterStatus.INACTIVE
        # DO NOT MESS WITH: threading lock/event handle state transition
        self._activateEvent = Event()
        # flag to test if first time Exporter activated
        self._first = True

    def activate(self):
        """
        Starts the extractor thread (if this is required)
        :return:
        """
        if self.get_exporter_status() == ExporterStatus.INACTIVE:
            # only want to start the thread once
            if self._first:
                self.start()
                self._first = False
            # thread you can now 'unpause'
            self._activateEvent.set()
            self.exporter_status = ExporterStatus.ACTIVE

    def deactivate(self):
        # if exporter is already INACTIVE, no need to do anything
        if self.get_exporter_status() == ExporterStatus.ACTIVE:
            # thread you are now 'magically' paused
            self._activateEvent.clear()
            self.exporter_status = ExporterStatus.INACTIVE

    def terminate(self):
        if not self._activateEvent.isSet():
            self._activateEvent.set()
        self.exporter_status = ExporterStatus.TERM

    def dissemination_rate_adjustment(self, data: dict):
        # TODO dissemination adaptiveness
        return data

    def update(self, data: dict):
        """
        Updates (prunes) the data based on specific logic (adaptive dissemination)
        :param data:
        :return:
        """
        if self.exporter_status == ExporterStatus.ACTIVE:
            data = self.dissemination_rate_adjustment(data)
        return data

    def run(self):
        pass

    def get_name(self):
        return self.name

    def set_name(self, name):
        self.name = name

    def get_exporter_status(self):
        """method that returns probe status... INACTIVE, ACTIVE, TERM
        """
        return self.exporter_status

    def set_exporter_status(self, status):
        if ExporterStatus.__contains__(status):
            self.exporter_status = status
        else:
            raise CatascopiaExporterStatusException('Exporter ' + self.name + ', attempted to set invalid status')


class ExporterStatus:
    typeNum = 3
    INACTIVE, ACTIVE, TERM = range(3)
    _typeStrings = {0: 'INACTIVE',
                    1: 'ACTIVE',
                    2: 'TERM'
                    }

    @staticmethod
    def __contains__(t):
        return False if (t not in range(ExporterStatus.typeNum)) else True

    @staticmethod
    def type_as_string(t):
        return ExporterStatus._typeStrings.get(t)


class CatascopiaExporterStatusException(Exception):
    pass



"""
This module implements the ignite exporter for the storage layer

"""
import json
import logging
import os
import threading
import time

import requests
from RainbowMonitoringSDK.exporters.Exporter import Exporter


class IgniteExporter(Exporter):
    """
    With this class the agent requests other service(s) through socket connection
    """

    connection_timer = 5
    ignite_request_has_failed = True

    def __init__(self, hostname, port, metric_repr=None):
        self.hostname, self.port, self.metric_repr = hostname, port, metric_repr
        Exporter.__init__(self, "IgniteExporter")

    def update(self, data: dict):
        res = []
        for _, metrics in data.items():
            print(f"Metrics from {_} have {len(metrics)} datapoints")
            for metric_name, metric in metrics.items():
                metric_repr = self.__extract_metric_representation(metric)
                res.append(metric_repr)
        self.metric_repr = res
        self.send_data(res)

    def send_data(self, res):

        IgniteExporter.ignite_request_has_failed = True
        while IgniteExporter.ignite_request_has_failed:
            try:
                payload = {"monitoring": res}
                headers = {'Content-Type': 'text/plain'}
                response = requests.post("http://%s:%s/put" %
                                         (self.hostname, self.port), data=json.dumps(payload), headers=headers)
                logging.info("Ignite server returns %s with the following response: %s" %
                             (response.status_code, response.text))
                IgniteExporter.ignite_request_has_failed = False
            except:
                IgniteExporter.ignite_request_has_failed = True
                logging.error("Error at the API does not respond", exc_info=True)
                # wait for some seconds (< 5 min) to retry to search for ignite exporter
                logging.error("Waiting %s seconds to retry to search for Ignite Exporter" % IgniteExporter.connection_timer)
                time.sleep(IgniteExporter.connection_timer)
                # double the wait
                if IgniteExporter.connection_timer < 300:
                    IgniteExporter.connection_timer *= 2

    def __extract_metric_representation(self, metric):
        metric_repr = metric.to_dict()
        entity_id, entity_type = self.__extract_group_and_entity_from_name(metric.name)
        metric_repr["entityID"] = entity_id
        metric_repr["entityType"] = entity_type
        metric_repr["metricID"] = metric.name
        return metric_repr

    @staticmethod
    def __extract_group_and_entity_from_name(name):
        entity_type = 'FOG_NODE'
        entity_id = os.environ['NODE_ID']  # TODO Find a better way for that
        if '_CGROUP_' in name:
            entity_type = 'CONTAINER'
            index = name.find('_CGROUP_') + len('_CGROUP_')
            entity_id = name[index: index + 12]
        return entity_id, entity_type



import os
import utils
from unittest import TestCase
from unittest.mock import patch

from RainbowMonitoringSDK.controller import Controller
from RainbowMonitoringSDK.probes.probelib.NetDataProbe import NetDataProbe
from RainbowMonitoringSDK.probes.Probe import ProbeStatus
from RainbowMonitoringSDK.exporters.exporterlib.IgniteExporter import IgniteExporter
from RainbowMonitoringSDK.exporters.Exporter import ExporterStatus


class TestExporters(TestCase):

    def setUp(self):
        configs = {"sensing-units": {"DefaultMonitoring": {"periodicity": "2m"}},
                   "dissemination-units": {"IgniteExporter": {"hostname": "test", "port": "1999"}}}
        mock_node_id_environ = patch.dict(os.environ, {"NODE_ID": "test_node_id"})
        mock_node_id_environ.start()
        self.controller = Controller(configs)
        self.empty_controller = Controller()

    def test_instantiate_sensing_units(self):
        self.empty_controller.instantiate_sensing_units()
        self.assertEqual(len(self.empty_controller.sensing_units), 0, "Sensing Units should be empty")

        self.controller.instantiate_sensing_units()
        self.assertEqual(len(self.controller.sensing_units), 1, "Sensing Units should have one unit")
        self.assertEqual(type(self.controller.sensing_units["DefaultMonitoring"]), NetDataProbe,
                         "Sensing Units should be NetData")

    def test_instantiate_dissemination_units(self):
        self.empty_controller.instantiate_dissemination_units()
        self.assertEqual(len(self.empty_controller.dissemination_units), 0, "Dissemination Units should be empty")

        self.controller.instantiate_dissemination_units()
        self.assertEqual(len(self.controller.dissemination_units), 1, "Dissemination Units should have one unit")
        self.assertEqual(type(self.controller.dissemination_units["IgniteExporter"]), IgniteExporter,
                         "Dissemination Units should be IgniteExporter")

    @patch('subprocess.Popen')
    def test_start_sensing_units(self, mock_subproc_popen):
        self.controller.start_sensing_units()
        self.assertEqual(self.controller.sensing_units["DefaultMonitoring"].get_probestatus(), ProbeStatus.ACTIVE,
                         "ProbeStatus should be activated")
        self.controller.sensing_units["DefaultMonitoring"].terminate()

    def test_start_dissemination_units(self):
        self.controller.start_dissemination_units()
        self.assertEqual(self.controller.dissemination_units["IgniteExporter"].get_exporter_status(),
                         ExporterStatus.ACTIVE, "ExporterStatus should be activated")
        self.controller.dissemination_units["IgniteExporter"].terminate()




import unittest
import uuid
from queue import Queue
from unittest.mock import patch

from RainbowMonitoringSDK.probes.Metric import Metric
from RainbowMonitoringSDK.probes.Probe import Probe, ProbeStatus


class TestProbe(unittest.TestCase):

    @patch.multiple(Probe, __abstractmethods__=set())
    def setUp(self):
        self.basic_probe = Probe("test-probe", 5)
        self.metric = Metric("metric-1", "units", "desc")  # dummy metric for test

    def test_set_logging(self):
        self.assertFalse(self.basic_probe.logging)
        self.assertIsNone(self.basic_probe.logger)
        self.basic_probe.set_logging("test/logging")
        self.assertTrue(self.basic_probe.logging)
        self.assertIsNotNone(self.basic_probe.logger)


    def test_attach_queue(self):
        queue = self.basic_probe.attachQueue()
        self.assertEqual(queue.maxsize, 1000000)
        queue = self.basic_probe.attachQueue()
        test_queue = Queue(maxsize=1000000)
        queue = self.basic_probe.attachQueue(test_queue)
        self.assertEqual(test_queue, queue)

    def test_dettach_queue(self):
        self.basic_probe.attachQueue()
        self.basic_probe.dettachQueue()
        self.assertIsNone(self.basic_probe.queue)
    #
    def test_get_probeid(self):
        self.assertEqual(type(self.basic_probe.get_probeid()), uuid.UUID)

    def test_get_name(self):
        self.assertEqual(self.basic_probe.get_name(), "test-probe")
    #
    def test_set_name(self):
        self.basic_probe.set_name("test-probe-2")
        self.assertEqual(self.basic_probe.get_name(), "test-probe-2")

    #
    def test_get_periodicity(self):
        self.assertEqual(self.basic_probe.periodicity, 5)
    #
    def test_set_periodicity(self):
        self.basic_probe.set_periodicity(10)
        self.assertEqual(self.basic_probe.periodicity, 10)
    #
    def test_get_probestatus(self):
        self.assertEqual(self.basic_probe.get_probestatus(), ProbeStatus.INACTIVE)


    def test_set_probestatus(self):
        self.assertEqual(self.basic_probe.get_probestatus(), ProbeStatus.INACTIVE)
        self.basic_probe.set_probestatus(ProbeStatus.TERM)
        self.assertEqual(self.basic_probe.get_probestatus(), ProbeStatus.TERM)
        self.basic_probe.set_probestatus(ProbeStatus.INACTIVE)
        self.assertEqual(self.basic_probe.get_probestatus(), ProbeStatus.INACTIVE)
    #
    def test_add_metric(self):
        self.basic_probe.add_metric(self.metric)
        self.assertEqual({self.metric.get_name(): self.metric}, self.basic_probe.metrics)
    #
    def test_get_metric(self):
        self.assertEqual(dict(),self.basic_probe.get_metrics())
    #
    def test_get_metrics(self):
        self.assertEqual(dict(), self.basic_probe.get_metrics())
    #
    def test_get_metrics_as_list(self):
        self.basic_probe.add_metric(self.metric)
        self.assertEqual([self.metric], list(self.basic_probe.get_metrics_as_list()))

    #
    def test_debugmode(self):
        self.assertFalse(self.basic_probe.get_debugmode())
        self.basic_probe.set_debugmode(True)
        self.assertTrue(self.basic_probe.get_debugmode())

    #
    def test_probe_lifecycle(self):
        self.basic_probe.activate()
        self.assertEqual(self.basic_probe.get_probestatus(), ProbeStatus.ACTIVE)
        self.basic_probe.deactivate()
        self.assertEqual(self.basic_probe.get_probestatus(), ProbeStatus.INACTIVE)
        self.basic_probe.terminate()
        self.assertEqual(self.basic_probe.get_probestatus(), ProbeStatus.TERM)
        self.basic_probe.run()

#






import json
import logging
import os
import subprocess
from functools import wraps
from time import time

from RainbowMonitoringSDK.probes.Metric import SimpleMetric, Metric
DOCKER_ID_LENGTH = 64

def get_entity_id():
    res = subprocess.Popen("head -1 /proc/self/cgroup|cut -d/ -f3", shell=True, stdout=subprocess.PIPE)
    encoded_first_line = res.stdout.readlines()[0].decode().replace("\n","")
    if len(encoded_first_line) != DOCKER_ID_LENGTH:
        logging.warning("This node probably is not containerised because has not container id")
        return None
    return encoded_first_line[:12]

class RainbowUtils:
    __ENTITY_ID__ = get_entity_id()
    __STORAGE_FILE__ = os.getenv("RAINBOW_METRIC_PATH", "./")+"rainbow.metrics.jsonl"
    __COUNT_IT_FUNCTIONS__ = {}

    @classmethod
    def store(cls, value, name, units, desc,
              minVal=float('-inf'), maxVal=float('inf'), higherIsBetter=True):
        simple_metric = SimpleMetric(name, units, desc, minVal, maxVal, higherIsBetter=higherIsBetter)
        simple_metric.set_val(value)
        cls.__store(simple_metric)

    @classmethod
    def __store(cls, metric: Metric):
        with open(cls.__STORAGE_FILE__, mode='a', encoding='utf-8') as file:
            new_name = cls.__container_enconded_name(metric)
            metric.set_name(new_name)
            metric_dict = metric.to_dict()
            json_metric = json.dumps(metric_dict) + '\n'
            file.write(json_metric)

    @classmethod
    def __container_enconded_name(cls, metric):
        if metric.name.startswith('_CGROUP_'):
            return metric.name
        prefix = '_CGROUP_' + cls.__ENTITY_ID__ + '__' if cls.__ENTITY_ID__ else ''
        new_name = prefix + metric.name  # Special name for
        return new_name

    @classmethod
    def timeit(cls, f):
        @wraps(f)
        def wrap(*args, **kw):
            ts = time()
            result = f(*args, **kw)
            te = time()
            name, unit, desc = "time__%s" % (f.__name__), 'ms', 'Execution duration from %s function' % (f.__name__)
            simple_metric = SimpleMetric(name, unit, desc, minVal=0.0, higherIsBetter=False)
            simple_metric.set_val(te - ts)
            cls.__store(simple_metric)
            return result

        return wrap

    @classmethod
    def export(cls, name, units, desc, minVal=float('-inf'), maxVal=float('inf'), higherIsBetter=True):
        def _export(f):
            simple_metric = SimpleMetric(name, units, desc, minVal=minVal, maxVal=maxVal, higherIsBetter=higherIsBetter)
            if not callable(f):
                simple_metric.set_val(f)
                cls.__store(simple_metric)
                return f

            @wraps(f)
            def wrap(*args, **kw):
                result = f(*args, **kw)
                if result is not None:
                    simple_metric.set_val(result)
                    cls.__store(simple_metric)
                else:
                    warning="Function %s returned None as result, monitoring can not capture this value" % f.__name__
                    logging.warning(warning)
                return result

            return wrap

        return _export

    @classmethod
    def countit(cls, f):
        @wraps(f)
        def wrap(*args, **kw):
            result = f(*args, **kw)
            functions, function_name = cls.__COUNT_IT_FUNCTIONS__, f.__name__
            if function_name not in functions:
                functions[function_name] = 0
            functions[function_name] += 1
            name, unit, desc = "countit__%s" % function_name, 'runs', 'The running times of %s function' % function_name
            simple_metric = SimpleMetric(name, unit, desc, minVal=0.0, higherIsBetter=False)
            simple_metric.set_val(functions[function_name])
            cls.__store(simple_metric)
            return result

        return wrap



from RainbowMonitoringSDK.exporters.exporterlib.APIcallExporter import APIcallExporter
# .exporterlib.KafkaExporter import KafkaExporter
from RainbowMonitoringSDK.exporters.exporterlib.MockExporter import MockExporter
from RainbowMonitoringSDK.exporters.exporterlib.IgniteExporter import IgniteExporter
from RainbowMonitoringSDK.exporters.exporterlib.RestAPIExporter import RestAPIExporter
from RainbowMonitoringSDK.exporters.Exporter import Exporter



import json
import logging
import os
import random
import time
from os.path import exists

import requests

from RainbowMonitoringSDK.probes.Metric import SimpleMetric, CounterMetric, DiffMetric, TimerMetric
from RainbowMonitoringSDK.probes.Probe import Probe


class WebAppMetricsProbe(Probe):

    def __init__(self,
                 periodicity, debug=False, _logging=False,
                 **kwargs):
        Probe.__init__(self, "WebAppLevelMetrics", periodicity,  debug, _logging )
        self.config = kwargs

    def get_desc(self):
        return "The monitoring probe of user defined metrics"

    def collect(self):
        json_metrics = self.retrieve_metrics()
        for _metric in json_metrics:
            metric = self.__metric_from_dictionary(_metric)
            self.add_metric(metric)

    @staticmethod
    def __metric_from_dictionary(collected):
        metric = SimpleMetric(collected.get('name'), collected.get('units'), collected.get('desc'))
        metric.set_val(collected.get('val'))
        metric.set_group(collected.get('group'))
        return metric

    def retrieve_metrics(self):
        res = []
        paths = self.config['sources']

        for path in paths:
            try:
                response = requests.get(path).json()
                for name, val in response.items():
                    res.append({
                        'name': name,
                        'val': float(val),
                        'units': 'NaN',
                        'desc': f'Metric for {name}.',
                        'group': 'WebAppMetricsProbe'
                        })
            except Exception as e:
                print(e)
        return res




    def get_metrics(self):
        self.collect()
        return Probe.get_metrics(self)



import json
import logging
import os
import random
import time
from os.path import exists
from RainbowMonitoringSDK.probes.Metric import SimpleMetric, CounterMetric, DiffMetric, TimerMetric
from RainbowMonitoringSDK.probes.Probe import Probe


class AppMetricsProbe(Probe):

    def __init__(self,
                 periodicity, debug=False, _logging=False,
                 **kwargs):
        Probe.__init__(self, "AppLevelMetrics", periodicity,  debug, _logging )
        self.config = kwargs

    def get_desc(self):
        return "The monitoring probe of user defined metrics"

    def collect(self):
        json_metrics = self.retrieve_metrics()
        for _metric in json_metrics:
            metric = self.__metric_from_dictionary(_metric)
            self.add_metric(metric)

    @staticmethod
    def __metric_from_dictionary(collected):
        metric = SimpleMetric(collected.get('name'), collected.get('units'), collected.get('desc'))
        metric.set_val(collected.get('val'))
        # metric.set_timestamp()
        metric.set_group(collected.get('group'))
        return metric

    def retrieve_metrics(self):
        res = []
        paths = self.config['sources']

        for path in paths:
            path_exists = path and exists(path)

            if path_exists and os.path.isdir(path):
                path = path + "rainbow.metrics.jsonl"
            path_exists = path and exists(path)

            if path_exists:
                set_of_metrics = self.__read_metrics_from_file(path)
                res.extend(set_of_metrics)
                self.__remove_file(path)
        return res

    @staticmethod
    def __read_metrics_from_file(path):
        res = []
        with open(path, 'r') as json_file:
            for line in json_file.readlines():
                try:
                    data = json.loads(line)
                    res.append(data)
                except Exception:
                    logging.error("An error occured at loading json line", exc_info=True)
                    continue
        return res

    @staticmethod
    def __remove_file(path):
        os.remove(path)

    def get_metrics(self):
        self.collect()
        return Probe.get_metrics(self)



from flask import Flask

from RainbowMonitoringSDK.exporters.Exporter import Exporter


class RestAPIExporter(Exporter):
    """
    API extractor class. This extractor provides an API where other services can request it
    """

    def __init__(self, port: int, path: str):
        self.port = port
        self.path = path
        self.data = {}
        self.app = Flask(__name__)
        self.__initialize_routes()
        Exporter.__init__(self, "RestAPIExporter")

    def __initialize_routes(self):
        @self.app.route(self.path)
        def main_route():
            print(self.data)
            return self.data

    def run(self):  # run a server that can accept requests
        self.app.run(host="0.0.0.0", port=self.port)

    def update(self, data: dict):
        data = Exporter.update(self, data)
        self.data.update(data)
        return data



import json

from kafka import KafkaProducer
from kafka.errors import KafkaError

from RainbowMonitoringSDK.exporters.Exporter import Exporter


class KafkaExporter(Exporter):
    """
    Kafka extractor class (TO TEST)
    """
    producer: KafkaProducer = None

    def __init__(self, brokers: str, topic: str = "metrics"):
        self.topic = topic
        self.brokers = brokers

    def run(self):  # creates the connection with Kafka
        self.producer = KafkaProducer(
            bootstrap_servers=self.brokers,
            key_serializer=str.encode,
            value_serializer=lambda m: json.dumps(m).encode('ascii')
        )

    def update(self, data: dict):
        data = Exporter.update(self, data)
        try:
            future = self.producer.send(self.topic, key=id, value=data)
            meta = future.get(timeout=10)
            return True
        except KafkaError as e:
            print(e)
        return False



import requests

from RainbowMonitoringSDK.exporters.Exporter import Exporter


class APIcallExporter(Exporter):
    """
    With this class the agent requests other service(s) at specific timestamps
    """

    def __init__(self, url: str,
                 method: str = "POST",
                 headers: dict = {},
                 auth: str = None,
                 password: str = None,
                 additional_params: dict = {}):
        self.url = url
        self.method = method
        self.headers = headers
        self.auth = auth
        self.password = password
        self.additional_params = additional_params

    def update(self, data: dict):
        try:
            data.update(self.additional_params)
            if self.method.lower() == 'get':
                res = self.__perform_get(data)
            elif self.method.lower() == 'post':
                res = self.__perform_post(data)
            else:
                res = self.__perform_other_methods(data)
            res.raise_for_status()
            return res
        except Exception as ex:
            print(ex)

    def __perform_get(self, data):
        return requests.get(
            self.url,
            auth=(self.auth, self.password) if self.auth and self.password else None,
            headers=self.headers,
            params=data
        )

    def __perform_post(self, data):
        return requests.post(
            self.url,
            auth=(self.auth, self.password) if self.auth and self.password else None,
            headers=self.headers,
            json=data
        )
    def __perform_other_methods(self, data):
        return getattr(requests, self.method.lower())(
            self.url,
            auth=(self.auth, self.password) if self.auth and self.password else None,
            headers=self.headers,
            json=data
        )



import os
from unittest import TestCase
from unittest.mock import patch

from RainbowMonitoringSDK.exporters.exporterlib.IgniteExporter import IgniteExporter
from RainbowMonitoringSDK.probes.Metric import Metric, SimpleMetric


# This method will be used by the mock to replace requests.post
def mocked_requests_post(*args, **kwargs):
    class MockResponse:
        def __init__(self, status_code, text):
            self.text = text
            self.status_code = status_code

    return MockResponse(200, "ok")


class TestExporters(TestCase):

    def setUp(self):
        self.hostname = "_test"
        self.port = '19999'
        self.exporter = IgniteExporter(self.hostname, self.port)
        mock_node_id_environ = patch.dict(os.environ, {"NODE_ID": "test_node_id"})
        mock_node_id_environ.start()

    # find a way to check input of post request and find the correct input
    @patch('requests.post', side_effect=mocked_requests_post)
    def test_update(self, mock_post):
        metric = SimpleMetric(name="_CGROUP_testing_name1", units="", desc="")
        metrics = {"metrics": {"metric1": metric}}
        with self.assertLogs() as captured:
            self.exporter.update(metrics)
        self.assertEqual(len(captured.records), 1)  # check that there is only one log message

    @patch('requests.post', side_effect=mocked_requests_post)
    def test_extract_metric_representation(self, mock_post):
        # case: entity is a fog node
        metric = SimpleMetric(name="testing_name1", units="", desc="")
        metrics = {"metrics": {"metric1": metric}}
        expected_result = [{'desc': '',
                            'entityID': 'test_node_id',
                            'entityType': 'FOG_NODE',
                            'metricID': 'testing_name1',
                            'name': 'testing_name1'
                            }]
        self.exporter.update(metrics)
        self.assertEqual(expected_result[0]["desc"], self.exporter.metric_repr[0]["desc"], "Wrong desc for fog node")
        self.assertEqual(expected_result[0]["entityID"], self.exporter.metric_repr[0]["entityID"], "Wrong entityId for fog node")
        self.assertEqual(expected_result[0]["entityType"], self.exporter.metric_repr[0]["entityType"], "Wrong entityType for fog node")
        self.assertEqual(expected_result[0]["metricID"], self.exporter.metric_repr[0]["metricID"], "Wrong metricID for fog node")
        self.assertEqual(expected_result[0]["name"], self.exporter.metric_repr[0]["name"], "Wrong testing_name1 for fog node")

        # case: entity is a container
        metric = SimpleMetric(name="_CGROUP_testing_name1", units="", desc="")
        metrics = {"metrics": {"metric1": metric}}
        self.exporter.update(metrics)
        expected_result = [{'desc': '',
                            'entityID': 'testing_name',
                            'entityType': 'CONTAINER',
                            'metricID': '_CGROUP_testing_name1',
                            'name': '_CGROUP_testing_name1'
                            }]
        self.assertEqual(expected_result[0]["desc"], self.exporter.metric_repr[0]["desc"], "Wrong desc for container")
        self.assertEqual(expected_result[0]["entityID"], self.exporter.metric_repr[0]["entityID"], "Wrong entityId for container")
        self.assertEqual(expected_result[0]["entityType"], self.exporter.metric_repr[0]["entityType"], "Wrong entityType for container")
        self.assertEqual(expected_result[0]["metricID"], self.exporter.metric_repr[0]["metricID"], "Wrong metricID for container")
        self.assertEqual(expected_result[0]["name"], self.exporter.metric_repr[0]["name"], "Wrong name for container")



import threading


class ThreadSafeDict(dict) :

    def __init__(self, * p_arg, ** n_arg) :
        dict.__init__(self, * p_arg, ** n_arg)
        self._lock = threading.Lock()

    def __enter__(self) :
        self._lock.acquire()
        return self

    def __exit__(self, type, value, traceback) :
        self._lock.release()

def time_to_seconds(input):
    if input.isnumeric(): return int(input)
    unit = input[-1].lower()
    if unit == 's': return int(input[:-1])
    if unit == 'm': return 60 * int(input[:-1])
    if unit == 'h': return 60 * 60 * int(input[:-1])
    raise Exception("The time unit is not defined")


from venv import logger

from RainbowMonitoringSDK.exporters.Exporter import Exporter


class MockExporter(Exporter):

    def __init__(self, **kwargs):
        print("**initilization of MockExporter with params: %s"%str(kwargs))


    def start(self):
        """
        Starts the extractor thread (if this is required)
        :return:
        """
        print("** start method of MockExporter is executed **")

    def dissemination_rate_adjustment(self, data: dict):
        print("** dissemination_rate_adjustment method of MockExporter is executed with %s data length **"%len(data))
        return data

    def update(self, data: dict):
        """
        Updates (prunes) the data based on specific logic (adaptive dissemination)
        :param data:
        :return:
        """
        print("** update method of MockExporter is executed with %s data **"%data)
        return data

    def run(self):
        print("** run method of MockExporter is executed**")
        pass




import unittest
from time import sleep

import requests
from RainbowMonitoringSDK.probes.Probe import ProbeStatus
from RainbowMonitoringSDK.probes.probelib.AppMetricsProbe import AppMetricsProbe
from unittest.mock import MagicMock, patch

from RainbowMonitoringSDK.probes.probelib.NetDataProbe import NetDataProbe
from requests.models import Response

def mocked_requests_get(*args, **kwargs):
    f = open("netdata_response.txt", "r")
    text = f.read()
    response = Response()
    response.status_code = 200
    response._content = str.encode(text)
    return response

class TestAppMetricsProbe(unittest.TestCase):

    def setUp(self):
        self.periodicity = 5
        self.prefix = "_test"
        self.ip_address = 'localhost'
        self.port = '19999'
        self.netdata_probe = NetDataProbe(periodicity=self.periodicity,
                                      prefix=self.prefix,
                                      ip_address=self.ip_address,
                                      port=self.port)

    @patch('requests.get', side_effect=mocked_requests_get)
    def test_collect(self, mock_get):
        self.netdata_probe.collect()
        self.assertGreater(len(self.netdata_probe.metrics),0)



import re
import subprocess
import requests
import yaml
from yaml.loader import SafeLoader
from RainbowMonitoringSDK.probes.Metric import SimpleMetric
from RainbowMonitoringSDK.probes.Probe import Probe
from RainbowMonitoringSDK.probes.utils import NetDataController


class NetDataProbe(Probe):
    # overall cpu, memomory, disk, disk i/o (ke gia kathe container)
    __REGEX_METRIC_NAME_FOR_DEMO = [
        '_SYSTEM_CPU_IDLE', '_SYSTEM_CPU_VISIBLETOTAL', '_APPS_SOCKETS_VISIBLETOTAL',
        '_SYSTEM_RAM_FREE', '_SYSTEM_RAM_USED', '_SYSTEM_RAM_CACHED', '_SYSTEM_RAM_BUFFERS',
        '_SYSTEM_RAM_VISIBLETOTAL',
        # Container metrics
        '_CGROUP_.{12}_CPU_SYSTEM', '_CGROUP_.{12}_CPU_SYSTEM',
        '_CGROUP_.{12}_CPU_VISIBLETOTAL', '_CGROUP_.{12}_MEM_VISIBLETOTAL'
    ]
    __NETDATA_URL_TEMPLATE = "http://%s:%s/api/v1/allmetrics"

    __NETDATA_STARTED = 0

    def __init__(self,
                 periodicity,
                 prefix="",
                 ip_address='0.0.0.0',
                 port='19999',
                 debug=False,
                 logging=False,
                 *args,
                 **kwargs):
        Probe.__init__(self, "NetData", periodicity, debug, logging)
        self.controller = NetDataController()
        self.ip_address = ip_address
        self.port = port
        self.url = self.__NETDATA_URL_TEMPLATE % (self.ip_address, self.port)
        self.prefix = prefix

    def get_desc(self):
        return "The monitoring probe of netdata"

    def collect(self):
        raw_metrics = self.retrieve_metrics()
        for raw_metric in raw_metrics:
            if not self.__is_in_demo_metrics(raw_metric['name']): continue
            metric = self.get_metric(raw_metric['name'])
            if not metric:
                metric = SimpleMetric(raw_metric['name'], raw_metric['unit'], raw_metric['name'])
                self.add_metric(metric)
            metric.set_val(raw_metric['value'])
            metric.set_group('NETDATA')  # TODO Change to be more group specific

    def __is_in_demo_metrics(self, metric_name):
        for regex in self.__REGEX_METRIC_NAME_FOR_DEMO:
            if re.search(regex, metric_name):
                return True
        # return False
        return True

    def retrieve_metrics(self):
        res = []
        try:
            metrics = requests.get(self.url).text
            self.__NETDATA_STARTED = 1
        except Exception as e:
            print("NetData is not working: ", e)
            return []
        for metric_line in metrics.splitlines():
            name, value, unit = self.__extract_information_from_line(metric_line)
            try:
                float(value)
            except Exception:
                continue
            res.append(dict(name=name, value=value, unit=unit))
        return res

    def __extract_information_from_line(self, metric_line):
        try:
            value_and_unit = metric_line.split("#")
            if len(value_and_unit) == 1:
                value, unit = value_and_unit[0], None
            else:
                value, unit = value_and_unit[0], value_and_unit[1]

            unit = unit.strip().replace(" ", "_")
            value = value.strip().split("=")
            name, value = value[0], value[1].replace('"', "")
            name = name.replace('NETDATA', self.prefix)
            return name, value, unit
        except Exception:
            return None, None, None

    def activate(self):
        # Open the file and load the file
        with open('/code/configs.yaml') as f:
            data = yaml.load(f, Loader=SafeLoader)
            self.controller.update_configs(data["sensing-units"])

        self.a_child_process = subprocess.Popen(["bash", "/usr/sbin/run.sh", "-c", "/code/RainbowMonitoringSDK/probes/net-data.cfg"], shell=False,
                                                stdout=open("netdata-stdout.txt", "w"),
                                                stderr=open("netdata-stderr.txt", "w"))
        Probe.activate(self)

    def cleanup(self):
        self.a_child_process.terminate()


def main():
    p = NetDataProbe("netdata", 5, ip_address="0.0.0.0", port=19999)
    p.set_debugmode(True)
    p.set_logging()

    p.activate()


if __name__ == "__main__":
    main()



import json
import logging
import os
import subprocess
from functools import wraps
from random import randint
from configupdater import ConfigUpdater
from time import time, sleep
from RainbowMonitoringSDK.probes.Metric import SimpleMetric, Metric

DOCKER_ID_LENGTH = 64


class ConfigFileIsNotValid(Exception):
    """Raised when the config file does not contain name"""
    pass


class NetDataController:
    disable_config_netdata = {
        "cpu": {
            "plugin:proc": ["/proc/stat"]
        },
        "memory": {
            "plugin:proc": ["/proc/meminfo"]
        },
        "disk": {
            "plugin:proc": ["/proc/diskstats"],
            "plugins": ["diskspace"]
        },
        "network": {
            "plugin:proc": [
                "/proc/net/dev", "/proc/net/wireless",
                "/proc/net/sockstat", "/proc/net/sockstat6",
                "/proc/net/netstat", "/proc/net/snmp",
                "/proc/net/snmp6", "/proc/net/sctp/snmp",
                "/proc/net/softnet_stat", "/proc/net/ip_vs/stats",
                "/proc/net/stat/conntrack", "/proc/net/stat/synproxy"
            ]
        }
    }

    set_config_netdata = {
        "cpu": "stat", "memory": "meminfo", "disk": "disk", "network": "net"
    }

    disable_configs = [
        "space usage", "enable", "inodes usage"
    ]

    def update_configs(self, config: dict = None):
        parser = ConfigUpdater()
        parser.read("/code/RainbowMonitoringSDK/probes/net-data.cfg")

        self.__set_periodicity(config, parser)
        disabled_groups = config["DefaultMonitoring"]["disabled-groups"]
        metric_groups = config["DefaultMonitoring"]["metric-groups"]

        # disable groups
        if disabled_groups is not None:
            for key in disabled_groups:
                print("disabled")
                print(self.disable_config_netdata[key].keys())
                for config_netdata_key in self.disable_config_netdata[key].keys():
                    for config_key in parser.sections():
                        if config_key == config_netdata_key:
                            # iterate in each subcategory the disabled category has
                            for key_category in self.disable_config_netdata[key].values():
                                if key_category in parser[config_netdata_key]:
                                    print(key_category)
                                    parser[config_netdata_key][key_category].value = "no"
                        if key in config_key and "plugin" in config_key:
                            for disabled_config in self.disable_configs:
                                if disabled_config in parser[config_key]:
                                    parser[config_key][disabled_config].value = "no"

        # override configurations
        if metric_groups is not None:
            for key in metric_groups:
                print("metric")
                if "name" in key:
                    for config_key in parser.sections():
                        # make key to lower case
                        key["name"] = key["name"].lower()
                        if self.set_config_netdata[key["name"]] in config_key and "plugin" in config_key:
                            print(config_key)
                            '''
                            warning!: if custom periodicity if smaller than general periodicity, 
                            then the general periodicity is applied
                            '''
                            if "periodicity" in key:
                                specific_periodicity = None
                                if "adaptive" in key["periodicity"]:
                                    if "minimum_periodicity" in key["params"]:
                                        specific_periodicity = key["params"]["minimum_periodicity"]
                                else:
                                    specific_periodicity = key["periodicity"]
                                if "update every" in parser[config_key]:
                                    parser[config_key]["update every"].value = specific_periodicity
                                else:
                                    parser[config_key]["update every"] = str(specific_periodicity)
                else:
                    raise ConfigFileIsNotValid()

        parser.update_file()

    def __set_periodicity(self, config: dict = None, parser=None):
        # set NetData periodicity
        periodicity = None
        if "periodicity" in config["DefaultMonitoring"]:
            periodicity = config["DefaultMonitoring"]["periodicity"]
        elif "general-periodicity" in config["general-periodicity"]:
            periodicity = config["general-periodicity"]
        if periodicity is not None:
            parser["global"]["update every"] = periodicity


def get_entity_id():
    res = subprocess.Popen("head -1 /proc/self/cgroup|cut -d/ -f3", shell=True, stdout=subprocess.PIPE)
    encoded_first_line = res.stdout.readlines()[0].decode().replace("\n", "")
    if len(encoded_first_line) != DOCKER_ID_LENGTH:
        logging.warning("This node probably is not containerised because has not container id")
        return None
    return encoded_first_line[:12]


class RainbowUtils:
    __ENTITY_ID__ = get_entity_id()
    __STORAGE_FILE__ = os.getenv("RAINBOW_METRIC_PATH", "./") + "rainbow.metrics.jsonl"
    __COUNT_IT_FUNCTIONS__ = {}

    @classmethod
    def store(cls, value, name, units, desc,
              minVal=float('-inf'), maxVal=float('inf'), higherIsBetter=True):
        simple_metric = SimpleMetric(name, units, desc, minVal, maxVal, higherIsBetter=higherIsBetter)
        simple_metric.set_val(value)
        cls.__store(simple_metric)

    @classmethod
    def __store(cls, metric: Metric):
        with open(cls.__STORAGE_FILE__, mode='a', encoding='utf-8') as file:
            new_name = cls.__container_enconded_name(metric)
            metric.set_name(new_name)
            metric_dict = metric.to_dict()
            json_metric = json.dumps(metric_dict) + '\n'
            file.write(json_metric)

    @classmethod
    def __container_enconded_name(cls, metric):
        if metric.name.startswith('_CGROUP_'):
            return metric.name
        prefix = '_CGROUP_' + cls.__ENTITY_ID__ + '__' if cls.__ENTITY_ID__ else ''
        new_name = prefix + metric.name  # Special name for
        return new_name

    @classmethod
    def timeit(cls, f):
        @wraps(f)
        def wrap(*args, **kw):
            ts = time()
            result = f(*args, **kw)
            te = time()
            name, unit, desc = "time__%s" % (f.__name__), 'ms', 'Execution duration from %s function' % (f.__name__)
            simple_metric = SimpleMetric(name, unit, desc, minVal=0.0, higherIsBetter=False)
            simple_metric.set_val(te - ts)
            cls.__store(simple_metric)
            return result

        return wrap

    @classmethod
    def export(cls, name, units, desc, minVal=float('-inf'), maxVal=float('inf'), higherIsBetter=True):
        def _export(f):
            simple_metric = SimpleMetric(name, units, desc, minVal=minVal, maxVal=maxVal, higherIsBetter=higherIsBetter)
            if not callable(f):
                simple_metric.set_val(f)
                cls.__store(simple_metric)
                return f

            @wraps(f)
            def wrap(*args, **kw):
                result = f(*args, **kw)
                if result is not None:
                    simple_metric.set_val(result)
                    cls.__store(simple_metric)
                else:
                    warning = "Function %s returned None as result, monitoring can not capture this value" % f.__name__
                    logging.warning(warning)
                return result

            return wrap

        return _export

    @classmethod
    def countit(cls, f):
        @wraps(f)
        def wrap(*args, **kw):
            result = f(*args, **kw)
            functions, function_name = cls.__COUNT_IT_FUNCTIONS__, f.__name__
            if function_name not in functions:
                functions[function_name] = 0
            functions[function_name] += 1
            name, unit, desc = "countit__%s" % function_name, 'runs', 'The running times of %s function' % function_name
            simple_metric = SimpleMetric(name, unit, desc, minVal=0.0, higherIsBetter=False)
            simple_metric.set_val(functions[function_name])
            cls.__store(simple_metric)
            return result

        return wrap


# Example of usage ---------------

@RainbowUtils.export("energy-consumption", "watt", "Energy consumption of the running node")
def energy_consumption_calculator():
    # current node's energy consumption
    return randint(1, 10)


@RainbowUtils.timeit
def intensive_workload_function():
    sleep(3)
    return randint(1, 10)


@RainbowUtils.countit
def http_endpoint():
    return randint(1, 10)


def main():
    variable = randint(1, 10)
    RainbowUtils.store(variable, "an_exported_variable", "some_unit",
                       "This variable is very crucial", minVal=float('-inf'),
                       maxVal=float('inf'), higherIsBetter=True)

    energy_consumption_calculator()
    intensive_workload_function()
    for times in range(20):
        http_endpoint()
        sleep(5)


if __name__ == "__main__":
    main()



import os
from unittest import TestCase
from unittest.mock import patch

from RainbowMonitoringSDK.exporters.exporterlib.RestAPIExporter import RestAPIExporter


# This method will be used by the mock to replace requests.post
def mocked_requests_post(*args, **kwargs):
    class MockResponse:
        def __init__(self, status_code, text):
            self.text = text
            self.status_code = status_code

    return MockResponse(200, "ok")


class TestExporters(TestCase):

    def setUp(self):
        self.path = "/path"
        self.port = 8080
        self.exporter = RestAPIExporter(self.port, self.path)
        self.app = self.exporter.app.test_client()
        mock_node_id_environ = patch.dict(os.environ, {"NODE_ID": "test_node_id"})
        mock_node_id_environ.start()

    @patch('requests.post', side_effect=mocked_requests_post)
    def test_update(self, mock_post):

        metrics = {"metric1": {"name": "testing_name1", "units": "", "desc": ""}}
        self.exporter.update(metrics)
        response = self.app.get(self.path, headers={"Content-Type": "application/json"})
        self.assertEqual(200, response.status_code)
        self.assertEqual(metrics, response.json)

        metrics_2 = {"metric2": {"name": "_CGROUP_testing_name1", "units": "", "desc": ""}}
        metrics.update(metrics_2)
        self.exporter.update(metrics)
        response = self.app.get(self.path, headers={"Content-Type": "application/json"})
        self.assertEqual(200, response.status_code)
        self.assertEqual(metrics, response.json)

        metrics_2 = {"metric2": {"name": "_CGROUP_testing_name2", "units": "", "desc": ""}}
        metrics.update(metrics_2)
        self.exporter.update(metrics)
        response = self.app.get(self.path, headers={"Content-Type": "application/json"})
        self.assertEqual(200, response.status_code)
        self.assertEqual(metrics, response.json)



import setuptools


with open("README.md", "r") as fh:
    long_description = fh.read()

reqs = ['requests', 'pyyaml', 'Flask', 'kafka', 'ConfigUpdater']

setuptools.setup(
    name="RainbowMonitoringSDK",
    version="0.0.2",
    author="Moysis Symeonides, Joanna Georgiou",
    author_email="symeonidis.moysis@cs.ucy.ac.cy, georgiou.joanna@cs.ucy.ac.cy",
    description="The Rainbow's Monitoring SDK library",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(where="code"),
    package_dir={'': 'code'},
    python_requires='>=3.5',
    install_requires=reqs
)



import unittest
from unittest import TestCase

from RainbowMonitoringSDK.exporters.Exporter import Exporter, ExporterStatus


class TestExporters(TestCase):

    def setUp(self):
        self.exporter = Exporter("test-exporter")

    def test_update(self):
        data = {"test": "test"}
        self.assertEqual(self.exporter.update(data), data,
                         "Wrong dissimination rate of data")

    def test_get_name(self):
        self.assertEqual(self.exporter.get_name(), "test-exporter")

    def test_set_name(self):
        self.exporter.set_name("test-exporter-2")
        self.assertEqual(self.exporter.get_name(), "test-exporter-2")

    def test_get_exporter_status(self):
        self.assertEqual(self.exporter.get_exporter_status(), ExporterStatus.INACTIVE)

    def test_set_exporter_status(self):
        self.assertEqual(self.exporter.get_exporter_status(), ExporterStatus.INACTIVE)
        self.exporter.set_exporter_status(ExporterStatus.TERM)
        self.assertEqual(self.exporter.get_exporter_status(), ExporterStatus.TERM)
        self.exporter.set_exporter_status(ExporterStatus.ACTIVE)
        self.assertEqual(self.exporter.get_exporter_status(), ExporterStatus.ACTIVE)

    def test_probe_lifecycle(self):
        self.exporter.activate()
        self.assertEqual(self.exporter.get_exporter_status(), ExporterStatus.ACTIVE)
        self.exporter.deactivate()
        self.assertEqual(self.exporter.get_exporter_status(), ExporterStatus.INACTIVE)
        self.exporter.terminate()
        self.assertEqual(self.exporter.get_exporter_status(), ExporterStatus.TERM)

